/// <summary>
/// ESChannelRequestResponse.cs
/// 
/// This sample demonstrates the RequestResponse() method of the
/// IEventServiceChannel class.
/// 
/// A ResponseListener will be registered to the IEventServiceChannel and the event
/// RequestEvent(0, "Hi There") will be sent to the correlator by calling the
/// IEventServiceChannel's RequestResponse() method and the ResponseEvent() 
/// is expected as a response.  
/// 
/// Requirement:
/// 
/// The RequestResponseEvent.mon must be injected to the correlator before
/// running this example.  Upon receiving the RequestEvent() event, the
/// RequestResponseEvent.mon monitorscript will send the response
/// (ResponseEvent()) back to the same channel and hence the callback of the 
/// ResponseListener will be called.
/// 
/// $Copyright(c) 2010,2012 Progress Software Corporation. All Rights Reserved.$ 
/// $Copyright (c) 2013, 2015 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
/// Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
/// 
/// </summary>
using System;

using Apama;
using Apama.Event;
using Apama.Event.Parser;
using Apama.Services.Event;
using Apama.Util;

public class ESChannelRequestResponse 
{
	private static readonly Logger logger = Logger.GetLogger(typeof(ESChannelRequestResponse));
	
	// Define the RequestEvent and ResponseEvent event type
	// NOTE: in order to use the RequestResponse() method, the Request
	//       and Response event must have a member  "messageId" 
	//		 (EventServiceChannelConstants.DefaultMessageIdFieldName) of 
	//       MonitorScript type integer, .NET type Int64.
	
	static readonly Field<long> messageIdField = FieldTypes.Integer.NewField(EventServiceChannelConstants.DefaultMessageIdFieldName);
	static readonly Field<string> msgField = FieldTypes.String.NewField("msg");
	static readonly Field<string> responseMsgField = FieldTypes.String.NewField("responseMsg");
	
	static readonly EventType requestEventType = new EventType("RequestEvent", messageIdField, msgField);
	static readonly EventType responseEventType = new EventType("ResponseEvent", messageIdField, responseMsgField);
	
		
	public void RunSample(string host, int port) 
	{
		try 
		{
			// get a IEventService instance from the EventServiceFactory

			using (IEventService eventService = EventServiceFactory.CreateEventService(host, port, "ESChannelRequestResponse sample"))
			{
	
				// Create the channel on "eventService.sample.channel"
				IEventServiceChannel ourChannel = eventService.AddChannel("eventService.sample.channel", null);
				
				// Register the RequestEventType to the channel
				ourChannel.RegisterEventType(requestEventType);
				
				// Create the requestEvent
				Event requestEvent = new Event("RequestEvent(0, \"Hi There\")");
				
				
				// Send two events in a for loop.  User can experiment
				// what will happen if responseWrapper.ReleaseLock() is NOT called
				// by changing the using(...) clause to an explicit call to 
				// responseWrapper.ReleaseLock or .Dispose() and commenting out 
				// that line
				for (int i = 0; i < 2; i++) 
				{
					// Now, make the requestReponse.
					// NOTE: RequestResponse() is a blocking call and won't return until
					//       a response is received (or other failure condition ocurred)
					
					Console.WriteLine("Sending RequestEvent() ...");
					
					using (IResponseWrapper responseWrapper = ourChannel.RequestResponse(requestEvent, responseEventType))
					{
						// print the response event
						Console.WriteLine("Response event received: " + responseWrapper.Event.GetField(responseMsgField));
					} // without using(), we would need to explicitly call Dispose()
				}
			}
			
		}
		catch (Exception ee) 
		{
			logger.Error("Exception caught: ", ee);
		}
	}
	
	public static void Main(string[] args)
	{
		if ((args.Length == 2) && (int.Parse(args[1]) > 0))
		{
			string host = args[0];
			int port = int.Parse(args[1]);
			new ESChannelRequestResponse().RunSample(host, port);
		}
		else
		{
			Console.WriteLine("Usage: ESChannelRequestResponse.exe <host> <port>");
		}
	}

}
