/*
 * This simple example illustrates how to use the .Java engine client to 
 * interface with Apama. The example connects to a remote Event Correlator, 
 * creates an event consumer, creates some MonitorScript code and injects it, 
 * injects some sample events, and receives some back from the Correlator 
 * when the monitor triggers. It then disconnects from the Correlator and 
 * exits.
 *
 * $Copyright(c) 2002,  2004-2006, 2012-2013 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013-2015, 2022 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 *
 */


import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import com.apama.engine.EngineStatus;
import com.apama.engine.MonitorScript;
import com.apama.engine.beans.EngineClientFactory;
import com.apama.engine.beans.interfaces.ConsumerOperationsInterface;
import com.apama.engine.beans.interfaces.EngineClientInterface;
import com.apama.event.Event;
import com.apama.event.EventListenerAdapter;
import com.apama.event.parser.EventParser;
import com.apama.event.parser.EventType;
import com.apama.event.parser.Field;
import com.apama.event.parser.FieldTypes;
import com.apama.util.Logger;


public class EngineClientSample 
{
	private static final Logger LOGGER = Logger.getLogger(EngineClientSample.class);

	static final Field<String> FIELD_TEXT = FieldTypes.STRING.newField("text");
	static final EventType EVENT_TEST_EVENT = new EventType("TestEvent", FIELD_TEXT);
	static final EventParser EVENT_PARSER = new EventParser(EVENT_TEST_EVENT);

	private void runSample(String host, int port) throws Exception
	{
		try
		(
			// Create the engine client (does not connect until first method requiring remove connection is invoked)
			// EngineClientInterface.close() will automatically get called upon exiting the try-with-resources block
			EngineClientInterface engineClient = EngineClientFactory.createEngineClient(host, port, "my-sample-process");
		) {
			// Listen for events sent to the "samplechannel" channel
			ConsumerOperationsInterface myConsumer = engineClient.addConsumer("myConsumer", "samplechannel");
			myConsumer.addEventListener(new EventListenerAdapter() {
				@Override
				public void handleEvent(Event evt) {
					evt = EVENT_PARSER.parse(evt.getText());
					System.out.println("Event listener received event with message: '"+evt.getField(FIELD_TEXT)+"'"); 
				}
			});
			
			// Optionally can monitor the connectedness of named consumers
			myConsumer.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent event) {
					if (ConsumerOperationsInterface.PROPERTY_CONSUMER_CONNECTED.equals(event.getPropertyName())) {
						ConsumerOperationsInterface theConsumer = (ConsumerOperationsInterface) event.getSource();
						
						boolean connected = ((Boolean) event.getNewValue()).booleanValue();
						System.out.println("Property changed for consumer \"" + theConsumer.getName() 
							+ "\": now "+ (connected ? "connected" : "disconnected"));
					}
				}
			});

			// Inject some MonitorScript
			MonitorScript epl = new MonitorScript(
				"event TestEvent {" +"\n" +
				"	string text;" +"\n" +
				"}" +"\n" +
				"" +"\n" +
				"monitor Echo {" +"\n" +
				"" +"\n" +
				"	TestEvent test;" +"\n" +
				"" +"\n" +
				"	action onload {" +"\n" +
				"		on all TestEvent(*):test {" +"\n" +
				"			send TestEvent(test.text) to \"samplechannel\";" +"\n" +
				"		}" +"\n" +
				"	}" +"\n" +
				"}");
			engineClient.injectMonitorScript(epl);

			// Wait a few seconds to be sure the injection event has been processed
			Thread.sleep(3000);

			// Get and display status statistics
			EngineStatus status = engineClient.getRemoteStatus();
			System.out.println(status);

			// Send some events (in a real app we might use the EventType class 
			// to construct these programatically)
			engineClient.sendEvents(
				new Event(EVENT_TEST_EVENT).setField(FIELD_TEXT, "Hello, World"),
				new Event(EVENT_TEST_EVENT).setField(FIELD_TEXT, "Welcome to Apama")
			);

			Thread.sleep(3000);

			// Delete the event type and monitor we added
			engineClient.deleteName("Echo", false);
			engineClient.deleteName("TestEvent", false);

			// Wait a few seconds for the output event to be received and the deletions processed
			Thread.sleep(3000);

			// Display status again
			status = engineClient.getRemoteStatus();
			System.out.println();
			System.out.println(status);
		}
	}

	public static void main(String[] argv) 
	{
		if ((argv.length != 2) || (Integer.parseInt(argv[1]) <= 0)) {
			// Bad command line given
			System.out.println("Usage: java EngineClientSample <host> <port>");
			return;
		}

		String host = argv[0];
		int port = Integer.parseInt(argv[1]);

		try {
			new EngineClientSample().runSample(host, port);
		} catch (Exception ex) {
			LOGGER.error("Sample failed: ", ex);
		}
	} 

}
